function [dOut, p, dExtra, pExtra] = densnd(x, cols, h, extra)

% DENSND(X)
% DENSND(X, COLS)
% DENSND(X, COLS, H)
% DENSND(X, COLS, H, EXTRA)
% 
% [D_OUT, P_OUT, D_EXTRA, P_EXTRA] = DENSND(...)
% 	
% X is an array of points (one row per point, one column per dimension)
% 
% COLS is the array of dimensions to actually take notice of (if you omit
% it, or set it to [], then all of the columns of X are used).
% 
% H is a smoothing parameter for the LENGTH(COLS)-dimensional Gaussian
% kernel used for smoothing. If it is omitted or set to [], a reasonable
% (non-parametrically derived) value is supplied.
% 
% EXTRA is an optional array of points in the same coordinates as X. These
% points will be compared against the distribution of X, but will not
% actually be included in the calculation of density: effectively, they are
% points at which to sample the density estimate.
% 
% D_OUT contains density values for each of the points.
% P_OUT contains, for each D_OUT value, the proportion of the whole set
% of D_OUT values that are greater than it (N.B. the "proportion" is actually
% computed with (number_of_points + 1) as the denominator). Thus, P_OUT(i) is
% an estimate of 1 - the statistical significance of X(i, :) against the whole
% set X.
% 
% D_EXTRA and P_EXTRA contain corresponding density and significance values
% evaluated at points EXTRA, if supplied.
% 
% If no output arguments are requested, a pretty picture is plotted.
% 
% Example:
%    x = randn(1000, 2);
%    densnd(x, [], [], [1, -2])
% Compares the point (1, -2), marked with a star, with a random distribution.

% Part of PFCMP version 2.5.41, an extension to the PSIGNIFIT toolbox for MATLAB versions 5 and up.
% The license and NO WARRANTY statement for PSIGNIFIT also applies to this release - see psych_legal.m
% Copyright (c) J.Hill 2002.
% mailto:psignifit@bootstrap-software.org
% http://www.bootstrap-software.org/psignifit/

if nargin < 4, extra = []; end
if nargin < 3, h = []; end
if nargin < 2, cols = []; end

if isempty(cols), cols = 1:size(x, 2); end
cols = unique(cols(:)');
x = x(:, cols);
x(any(isnan(x), 2), :) = [];
if ~isempty(extra)
	extra = extra(:, cols);
	extra(any(isnan(extra), 2), :) = [];
end

if ~isreal(x), error('distribution cannot contain complex elements'), end

n = size(x, 1);
nExtra = size(extra, 1);

if isempty(h), h = 0.65; end % 100 /(0.68 * n); end
if length(h) == 1, h = h * 0.5 * diff(quantile(x, [0.16;0.84]), 1); end
h = h(:)';
if length(h) ~= size(x, 2), error('H must be a scalar, or a vector with one value per column of X'), end
xn = x ./ repmat(h, n, 1);
if ~isempty(extra), epn = extra ./ repmat(h, nExtra, 1); end

d = nan * ones(n + nExtra, 1);
for i = 1:(n + nExtra)
	if i <= n, xi = xn(i, :); else xi = epn(i - n, :); end
	xi = repmat(xi, n, 1);
	xi = xn - xi;
	xi = xi.^2;
	xi = sum(xi, 2);
	xi = exp(-0.5 * xi);
	xi(isinf(xi) | isnan(xi)) = [];
	if ~isempty(xi), d(i) = sum(xi); end
end
d = d / n;
dExtra = d(n+1:end);
d(n+1:end) = [];

[ds i] = sort(d);
p(i) = 1 - (1:n)/(n+1); p = p(:);
pExtra = 1 - cpe(dExtra, d); pExtra = pExtra(:);

if nargout
	dOut = d;
elseif length(cols)==2
	colormap(jet(256))

%% 	PATCH VERSION (avoid @^%$@ crash)
	cla
	xlim(sensiblerange(x(:, 1), [-4 4]))
	ylim(sensiblerange(x(:, 2), [-4 4]))
	hold on
 	dh = patchplot(x, p, 'size', 2);
	if ~isempty(extra)
%%		DOT VERSION
		extraH = dotplot(extra, pExtra, 'marker', 'p', 'markersize', 16);
		set(extraH, {'markeredgecolor'}, contrastingcolour(get(extraH, {'markerfacecolor'})))
%%		PATCH VERSION
% 		extraH = patchplot(extra, pExtra, 'size', 5, 'vertices', 3);
% 		cm = colormap;
% 		colours = contrastingcolour(cm(1 + round((size(cm, 1)-1) * pExtra), :));
	end
	hold off

%%	DOT VERSION	
% 	dh = dotplot([x;extra], [p;pExtra]);
% 	extraH = dh(n+1:end);
% 	if ~isempty(extraH)
% 		set(extraH, 'marker', 'p', 'markersize', 16)
% 		set(extraH, {'markeredgecolor'}, contrastingcolour(get(extraH, {'markerfacecolor'})))
% 	end
	pbaspect auto
	
	cb = colorbar;
	set(cb, 'ylim', [0 1])
	pt = get(cb, 'children');
	set(pt, 'ydata', [0 1]);
	ti = get(cb, 'title');
	set(ti, 'string', 'p')
	pbaspect(cb, 'auto')
	figure(gcf)
end
