function [mfc, mec, colours] = markercolour(h, faceLightness, edgeLightness, colours)
% [MFC MEC COL] = MARKERCOLOUR(H)
% 	For LINE handles H, gets the 'markerfacecolor', 'markeredgecolor' and 'color'
% 	properties, returning them as M-by-3 arrays MFC, MEC and COL respectively.
% 	
% MARKERCOLOUR(H, FL, [EL], [COL])
% 	Sets the 'color' property of LINE handles H to COL.
% 	Then sets the 'markerfacecolor' according to the face lightness	argument FL,
% 	Then sets the 'markeredgecolor' according to the edge lightness argument EL.
% 	Lightness is a number from 0 (black) to 1 (white), where 0.5 corresponds to
% 	the colour in the 'color' property.
% 	
% 	Pass [] in FL, EL or COL in order to leave the corresponding property unchanged.
% 
% E.g:
% 	MARKERCOLOUR(H, 0.5, 0.5)
% 		sets marker face and edge colour for each handle in H to be the same colour
% 		as the line.
% 	MARKERCOLOUR(H, 0.75, 0.25, [0 0 1])
% 		blue lines, pale blue marker faces, dark blue marker edges.

% Part of PFCMP version 2.5.41, an extension to the PSIGNIFIT toolbox for MATLAB versions 5 and up.
% The license and NO WARRANTY statement for PSIGNIFIT also applies to this release - see psych_legal.m
% Copyright (c) J.Hill 2002.
% mailto:psignifit@bootstrap-software.org
% http://www.bootstrap-software.org/psignifit/


if nargin < 4, colours = []; end
if nargin < 3, edgeLightness = []; end
if nargin < 2, faceLightness = []; end

if prod(size(faceLightness)) == 1, faceLightness = repmat(faceLightness, size(h)); end
if prod(size(edgeLightness)) == 1, edgeLightness = repmat(edgeLightness, size(h)); end
h = h(:);
faceLightness = faceLightness(:);
edgeLightness = edgeLightness(:);

if ~isempty(colours), setcolour(h, 'color', colours), end
colours = getcolour(h, 'color');
if ~isempty(faceLightness), setcolour(h, 'markerfacecolor', mixcolour(colours, faceLightness)), end
if ~isempty(edgeLightness), setcolour(h, 'markeredgecolor', mixcolour(colours, edgeLightness)), end
if nargout > 0 | nargin < 2
	mfc = getcolour(h, 'markerfacecolor');
	mec = getcolour(h, 'markeredgecolor');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%
function c = mixcolour(c, lg)

if any(size(lg) ~= size(c) & size(lg) ~= 1)
	error('dimension mismatch')
end
darken = (lg < 0.5);
coefft(darken, 1) = 2 * lg(darken);
coefft(~darken, 1) = 2 * (1 - lg(~darken));
offset(darken, 1) = 0;
offset(~darken, 1) = 1 - coefft(~darken);
coefft = repmat(coefft, size(c)./size(coefft));
offset = repmat(offset, size(c)./size(offset));
c = offset + coefft .* c;

%%%%%%%%%%%%%%%%%%%%%%%%%%%
function setcolour(h, propname, c)

if isempty(h), return, end
h = h(:);
if size(c, 1) == 1, c = repmat(c, length(h), 1); end
if size(c, 1) ~= length(h), error('must supply one row, or one row per handle'), end
if ~iscell(c), c = num2cell(c, 2); end
if ~iscell(propname), propname = {propname}; end
if size(c, 2) ~= size(propname, 2), error('must supply one set of colours per property name'), end
set(h, propname, c)

%%%%%%%%%%%%%%%%%%%%%%%%%%%
function cOut = getcolour(h, propname)

if isempty(h), cOut = []; return, end
h = h(:);
if ~iscell(propname), propname = {propname}; end
c = get(h, propname);
for i = 1:size(c, 2), cOut{i} = cat(1, c{:, i}); end
if size(cOut, 1) == 1, cOut = cOut{1}; end


